"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var parse_1 = require("./parse");
var units_1 = require("./lib/units");
var parseDate_1 = require("./lib/parseDate");
var apply_1 = require("./apply");
/**
 * Convert the input value to milliseconds represented by a `Time` object.
 * If a number is passed this is returned verbatim as the number
 * of milliseconds.
 *
 * @example toMilliseconds({ days: 1 }) // 86400000
 */
exports.toMilliseconds = function (duration, referenceDate) {
    if (referenceDate !== undefined) {
        return apply_1.apply(referenceDate, duration).getTime() - parseDate_1.parseDate(referenceDate).getTime();
    }
    var parsedTime = parse_1.parse(duration);
    return units_1.UNITS.reduce(function (total, _a) {
        var unit = _a.unit, milliseconds = _a.milliseconds;
        return total + (parsedTime[unit] * milliseconds);
    }, 0);
};
var createTimeConverter = function (unit) {
    return function (duration, referenceDate) {
        return exports.toMilliseconds(duration, referenceDate) / units_1.UNITS_MAP[unit].milliseconds;
    };
};
/**
 * Convert the input value to seconds.
 * @example toSeconds({ minutes: 2 }) // 120
 */
exports.toSeconds = createTimeConverter('seconds');
/**
 * Convert the input value to minutes.
 * @example toMinutes({ hours: 1, minutes: 10 }) // 70
 */
exports.toMinutes = createTimeConverter('minutes');
/**
 * Convert the input value to hours.
 * @example toHours({ days: 1 }) // 24
 */
exports.toHours = createTimeConverter('hours');
/**
 * Convert the input value to days.
 * @example toDays({ hours: 12 }) // 0.5
 */
exports.toDays = createTimeConverter('days');
/**
 * Convert the input value to weeks.
 * @example toWeeks({ days: 14 }) // 2
 */
exports.toWeeks = createTimeConverter('weeks');
/**
 * Convert the input value to months.
 * Note, this is a rough approximation as the length of a month is variable.
 *
 * @example toMonths({ months: 10, days: 365 }) // 11
 */
exports.toMonths = createTimeConverter('months');
/**
 * Convert the input value to years.
 * Note, this is a rough approximation as the length of a year is variable.
 *
 * @example toYears({ years: 10, days: 365 }) // 11
 */
exports.toYears = createTimeConverter('years');
//# sourceMappingURL=toUnit.js.map